
' BASIC Chess
' UCI engine based on Dean Menezes QBASIC chess program
' Roland Chastain 2018

#include once "log.bi"
#include once "fen.bi"
#include once "uci.bi"
#include once "utils.bi"
#include once "version.bi"

declare sub STARTPOS()
declare sub MAKEMOVE overload(A as const integer, B as const integer, X as const integer, Y as const integer)
declare sub MAKEMOVE(S as const string)
declare sub SAVEBD(BD() as integer)
declare sub RESTOREBD(BD() as integer)
declare sub SHOWBD()
declare function MAN(A as const integer, B as const integer) as string
declare sub IO(A as const integer, B as const integer, X as const integer, Y as const integer, RESULT as const integer)
declare function EVALUATE(ID as const integer, PRUNE as const integer) as integer
declare sub MOVELIST(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub BISHOP(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub KING(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub KNIGHT(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub PAWN(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub QUEEN(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare sub ROOK(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
declare function INCHECK() as boolean

dim shared as integer BOARD(0 to 7, 0 to 7)
dim shared as integer BESTA(0 to 7), BESTB(0 to 7), BESTX(0 to 7), BESTY(0 to 7)
dim shared as integer LEVEL, MAXLEVEL, SCORE, CFLAG
dim shared as integer activeColor

CFLAG = 0
LEVEL = 0
MAXLEVEL = 5

StartPosition:
data  -500, -270, -300, -900,-5000, -300, -270, -500
data  -100, -100, -100, -100, -100, -100, -100, -100
data     0,    0,    0,    0,    0,    0,    0,    0
data     0,    0,    0,    0,    0,    0,    0,    0
data     0,    0,    0,    0,    0,    0,    0,    0
data     0,    0,    0,    0,    0,    0,    0,    0
data   100,  100,  100,  100,  100,  100,  100,  100
data   500,  270,  300,  900, 5000,  300,  270,  500

sub STARTPOS()
  dim Z as integer
  restore StartPosition
  for Y as integer = 0 to 7
    for X as integer = 0 to 7
      read Z
      BOARD(Y, X) = Z
    next X
  next Y
  activeColor = 1
end sub

sub MAKEMOVE(A as const integer, B as const integer, X as const integer, Y as const integer)
  dim piececolor as integer = Sgn(BOARD(B, A))
  
  if (piececolor * BOARD(B, A) = 100) andalso (X <> A) andalso (BOARD(Y, X) = 0) then
    BOARD(Y, A) = 0
  end if
  
  BOARD(Y, X) = BOARD(B, A)
  BOARD(B, A) = 0
  
  if ((Y = 0) or (Y = 7)) and (piececolor * BOARD(Y, X) = 100) then BOARD(Y, X) *= 9
  
  if (piececolor * BOARD(Y, X) = 5000) andalso (abs(X - A) = 2) then
    select case X
      case 6
        BOARD(Y, 5) = BOARD(Y, 7)
        BOARD(Y, 7) = 0
      case 2
        BOARD(Y, 3) = BOARD(Y, 0)
        BOARD(Y, 0) = 0
    end select
  end if
end sub

sub MAKEMOVE(S as const string)
  dim as integer A, B, X, Y
  StrToCoord(S, A, B, X, Y)
  MAKEMOVE(A, B, X, Y)
end sub

sub SAVEBD(BD() as integer)
  for Y as integer = 0 to 7
    for X as integer = 0 to 7
      BD(Y, X) = BOARD(Y, X)
    next X
  next Y
end sub

sub RESTOREBD(BD() as integer)
  for Y as integer = 0 to 7
    for X as integer = 0 to 7
      BOARD(Y, X) = BD(Y, X)
    next X
  next Y
end sub

sub SHOWBD
  dim ln as string
  dim row as string
  Print("info +   A B C D E F G H   +")
  Print("info")
  for B as integer = 0 to 7
    row = Chr(Asc("8") - B)
    ln = "info " & row & "   "
    for A as integer = 0 to 7
      ln += MAN(A, B) & " "
    next
    Print ln & "  " & row
  next
  Print("info")
  Print("info +   A B C D E F G H   +")
end sub

function MAN(A as const integer, B as const integer) as string
  dim symb as string = "."
  dim as integer PIECE = int(abs(BOARD(B, A)))
  if PIECE = 100 then symb = "P"
  if PIECE = 270 then symb = "N"
  if PIECE = 300 then symb = "B"
  if PIECE = 500 then symb = "R"
  if PIECE = 900 then symb = "Q"
  if PIECE = 5000 or PIECE = 7500 then symb = "K"
  if board(b, a) < 0 then symb = lcase(symb)
  return symb
end function

sub IO(A as const integer, B as const integer, X as const integer, Y as const integer, RESULT as const integer)
  dim as integer XX(0 to 26), YY(0 to 26), PIECE = 0, NDX = -1, MOVER, TARGET
  dim as string cmd, fen, ss()

  dim posdata as TPositionData
  dim as integer c, d
  dim as integer TABLE(-6 to 6) = {-7500, -900, -500, -300, -270, -100, 0, 100, 270, 300, 500, 900, 7500}
  
  if A >= 0 then
    Print("bestmove " & CoordToStr(A, B, X, Y))
    MAKEMOVE(A, B, X, Y)
    activeColor = -1 * activeColor
  end if
  
  do
    input "", cmd
    ToLog(cmd)
    
    if uci.IsCommandQuit(cmd) then
      CloseLog()
      end
      
    elseif uci.IsCommandUCI(cmd) then
      Print("id name " & PROGRAM_NAME & " " & PROGRAM_VERSION)
      Print("id author " & AUTHOR_NAME)
      Print("uciok")
      
    elseif uci.IsCommandReady(cmd) then
      Print("readyok")
      
    elseif uci.IsCommandNewGame(cmd)  then
      STARTPOS()
      
    elseif uci.IsCommandPosition(cmd) then
      
      if uci.IsCommandStartPos(cmd) then
        STARTPOS()
      elseif uci.IsCommandFEN(cmd) then
        if uci.GetFEN(cmd, fen) then
          posdata = ParseFEN(cmd)
          for d = 0 to 7
            for c = 0 to 7
              BOARD(d, c) = TABLE(posdata.board(1 + c, 8 - d))
            next
          next
          activeColor = posdata.activeColor
        end if
      end if
      if uci.IsCommandMoves(cmd) then
        if uci.GetMoves(cmd, ss()) then
          for i as integer = lbound(ss) to ubound(ss)
            MAKEMOVE(ss(i))
            activeColor = -1 * activeColor
          next i
        end if
      end if
      SHOWBD()
      
    elseif uci.IsCommandGo(cmd) then
      exit sub
    
    end if
  loop
end sub

function EVALUATE(ID as const integer, PRUNE as const integer) as integer
  dim as integer XX(0 to 26), YY(0 to 26)
  LEVEL = LEVEL + 1
  dim as integer BESTSCORE = 10000 * ID
  dim as integer NDX = -1, X, Y, OLDSCORE, MOVER, TARGET
  for B as integer = 7 to 0 step -1
    for A as integer = 7 to 0 step -1
      if sgn(BOARD(B, A)) <> ID then continue for
      MOVELIST(A, B, XX(), YY(), NDX)
      for I as integer = 0 to NDX
        X = XX(I)
        Y = YY(I)
        if LEVEL = 1 then
          Print "info trying " & CoordToStr(A, B, X, Y)
        end if
        OLDSCORE = SCORE
        MOVER = BOARD(B, A)
        TARGET = BOARD(Y, X)
        MAKEMOVE(A, B, X, Y)
        if (LEVEL < MAXLEVEL) then SCORE = SCORE + EVALUATE(-ID, BESTSCORE - TARGET + ID * (8 - abs(4 - X) - abs(4 - Y)))
        SCORE = SCORE + TARGET - ID * (8 - abs(4 - X) - abs(4 - Y))
        if (ID < 0 and SCORE > BESTSCORE) or (ID > 0 and SCORE < BESTSCORE) then
          BESTA(LEVEL) = A
          BESTB(LEVEL) = B
          BESTX(LEVEL) = X
          BESTY(LEVEL) = Y
          BESTSCORE = SCORE
          if (ID < 0 and BESTSCORE >= PRUNE) or (ID > 0 and BESTSCORE <= PRUNE) then
            BOARD(B, A) = MOVER
            BOARD(Y, X) = TARGET
            SCORE = OLDSCORE
            LEVEL = LEVEL - 1
            EVALUATE = BESTSCORE
            exit function
          end if
        end if
        BOARD(B, A) = MOVER
        BOARD(Y, X) = TARGET
        SCORE = OLDSCORE
      next
    next
  next
  LEVEL = LEVEL - 1
  EVALUATE = BESTSCORE
end function

sub MOVELIST(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer PIECE = int(abs(BOARD(B, A)))
  NDX = -1
  if PIECE = 100 then
    PAWN(A, B, XX(), YY(), NDX)
  elseif PIECE = 270 then KNIGHT(A, B, XX(), YY(), NDX)
  elseif PIECE = 300 then BISHOP(A, B, XX(), YY(), NDX)
  elseif PIECE = 500 then ROOK(A, B, XX(), YY(), NDX)
  elseif PIECE = 900 then QUEEN(A, B, XX(), YY(), NDX)
  else KING(A, B, XX(), YY(), NDX)
  end if
end sub

sub BISHOP(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer ID = sgn(BOARD(B, A)), X, Y
#macro GOSUB3
if ID <> sgn(BOARD(Y, X)) then
  NDX = NDX + 1
  XX(NDX) = X
  YY(NDX) = Y
end if
#endmacro
  for DXY as integer = 1 to 7
    X = A - DXY
    Y = B + DXY
    if X < 0 or X > 7 or Y < 0 or Y > 7 then exit for
    GOSUB3
    if BOARD(Y, X) then exit for
  next
  for DXY as integer = 1 to 7
    X = A + DXY
    Y = B + DXY
    if X < 0 or X > 7 or Y < 0 or Y > 7 then exit for
    GOSUB3
    if BOARD(Y, X) then exit for
  next
  for DXY as integer = 1 to 7
    X = A - DXY
    Y = B - DXY
    if X < 0 or X > 7 or Y < 0 or Y > 7 then exit for
    GOSUB3
    if BOARD(Y, X) then exit for
  next
  for DXY as integer = 1 to 7
    X = A + DXY
    Y = B - DXY
    if X < 0 or X > 7 or Y < 0 or Y > 7 then exit for
    GOSUB3
    if BOARD(Y, X) then exit for
  next
end sub

sub KING(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer ID = sgn(BOARD(B, A))
  for DY as integer = -1 to 1
    if B + DY < 0 or B + DY > 7 then continue for'GOTO 12
    for DX as integer = -1 to 1
      if A + DX < 0 or A + DX > 7 then continue for'GOTO 11
      if ID <> sgn(BOARD(B + DY, A + DX)) then
        NDX = NDX + 1
        XX(NDX) = A + DX
        YY(NDX) = B + DY
      end if
    next
  next
end sub

sub KNIGHT(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer ID = sgn(BOARD(B, A)), X, Y
#macro GOSUB5
if (X >= 0) and (X <= 7) and (Y >= 0) and (Y <= 7) andalso (ID <> sgn(BOARD(Y, X))) then
  NDX = NDX + 1
  XX(NDX) = X
  YY(NDX) = Y
end if
#endmacro
  X = A - 1
  Y = B - 2
  GOSUB5
  X = A - 2
  Y = B - 1
  GOSUB5
  X = A + 1
  Y = B - 2
  GOSUB5
  X = A + 2
  Y = B - 1
  GOSUB5
  X = A - 1
  Y = B + 2
  GOSUB5
  X = A - 2
  Y = B + 1
  GOSUB5
  X = A + 1
  Y = B + 2
  GOSUB5
  X = A + 2
  Y = B + 1
  GOSUB5
end sub

sub PAWN(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer ID = sgn(BOARD(B, A))
  if (A - 1) >= 0 and (A - 1) <= 7 and (B - ID) >= 0 and (B - ID) <= 7 then
    if sgn(BOARD((B - ID), (A - 1))) = -ID then
      NDX = NDX + 1
      XX(NDX) = A - 1
      YY(NDX) = B - ID
    end if
  end if
  if (A + 1) >= 0 and (A + 1) <= 7 and (B - ID) >= 0 and (B - ID) <= 7 then
    if sgn(BOARD((B - ID), (A + 1))) = -ID then
      NDX = NDX + 1
      XX(NDX) = A + 1
      YY(NDX) = B - ID
    end if
  end if
  if A >= 0 and A <= 7 and (B - ID) >= 0 and (B - ID) <= 7 then
    if BOARD((B - ID), A) = 0 then
      NDX = NDX + 1
      XX(NDX) = A
      YY(NDX) = B - ID
      if (ID < 0 and B = 1) or (ID > 0 and B = 6) then
        if BOARD((B - ID - ID), A) = 0 then
          NDX = NDX + 1
          XX(NDX) = A
          YY(NDX) = B - 2 * ID
        end if
      end if
    end if
  end if
end sub

sub QUEEN(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  BISHOP(A, B, XX(), YY(), NDX)
  ROOK(A, B, XX(), YY(), NDX)
end sub

sub ROOK(A as const integer, B as const integer, XX() as integer, YY() as integer, byref NDX as integer)
  dim as integer ID = sgn(BOARD(B, A))
  
  for X as integer = A - 1 to 0 step -1
    if ID <> sgn(BOARD(B, X)) then
    NDX = NDX + 1
    XX(NDX) = X
    YY(NDX) = B
    end if
    if (BOARD(B, X)) then exit for
  next
  
  for X as integer = A + 1 to 7 step 1
    if ID <> sgn(BOARD(B, X)) then
    NDX = NDX + 1
    XX(NDX) = X
    YY(NDX) = B
    end if
    if (BOARD(B, X)) then exit for
  next
  
  for Y as integer = B - 1 to 0 step -1
    if ID <> sgn(BOARD(Y, A)) then
    NDX = NDX + 1
    XX(NDX) = A
    YY(NDX) = Y
    end if
    if (BOARD(Y, A)) then exit for
  next
  
  for Y as integer = B + 1 to 7 step 1
    if ID <> sgn(BOARD(Y, A)) then
    NDX = NDX + 1
    XX(NDX) = A
    YY(NDX) = Y
    end if
    if (BOARD(Y, A)) then exit for
  next
end sub

function INCHECK() as boolean
  dim as integer XX(27), YY(27), NDX, X, Y
  for B as integer = 0 to 7
    for A as integer = 0 to 7
      if activeColor * BOARD(B, A) >= 0 then continue for ' /!\
      MOVELIST(A, B, XX(), YY(), NDX)
      for I as integer = 0 to NDX step 1
        X = XX(I)
        Y = YY(I)
        if activeColor * BOARD(Y, X) = 5000 then ' /!\
          INCHECK = true
          exit function
        end if
      next
    next
  next
  INCHECK = false
end function
