
#include once "definitions.bi"
#include once "types.bi"
#include once "utils.bi"

const as byte cNothingValue = 000
const as byte cPawnValue    = 002
const as byte cBishopValue  = 006
const as byte cKnightValue  = 007
const as byte cRookValue    = 010
const as byte cQueenValue   = 019
const as byte cKingValue    = 126
const as byte cBorderValue  = 127

dim shared as byte cPawnMove  (1 to 4) = {+01, +02, -09, +11}
dim shared as byte cBishopMove(1 to 4) = {+11, -11, +09, -09}
dim shared as byte cRookMove  (1 to 4) = {-01, +01, -10, +10}
dim shared as byte cKnightMove(1 to 8) = {+12, +21, +19, +08, -12, -21, -19, -08}
dim shared as byte cKingMove  (1 to 8) = {-01, +01, -10, +10, +11, -11, +09, -09}

type TMoveRec
  as byte fromSquare
  as byte toSquare
  as short value
end type

type TMoveList
  as TMoveRec array(1 to moveListSize)
  as byte count
end type

type TBoard
  as byte board(-10 to 109)
  as byte castlingAvailable(e1g1 to e8c8)
  as byte castlingLicence(e1g1 to e8c8)
  as byte enPassantTargetSquare(black to white)
  as byte activeColor, oneKingCaptured
  as short halfmoveCount, materialBalance
  as TMoveList list
  declare sub Initialize(byval as string = "")
  declare sub MovePiece(byval as byte, byval as byte, byval as byte = cQueenValue)
  declare sub AddMoveToList(byval as byte, byval as byte)
  declare sub GenerateMoves()
  declare sub GenerateCastling(byval as tCastling)
  declare sub GenerateAllMoves()
  declare sub CheckCastlingLicence(byval as byte)
  declare function PositionValue(byval as byte) as short
  declare function Text() as string
  declare function Check() as byte
end type

sub TBoard.Initialize(byval s as string = "")
  dim as byte x, y, z
  
  for z = -10 to 109
    board(z) = cBorderValue
  next z
  
  if Len(s) = 0 then
    restore DefaultPlacement
    for y = 8 to 1 step -1
      for x = 1 to 8
        read board(10 * x + y)
      next x
    next y
    castlingAvailable(e1g1) = true
    castlingAvailable(e1c1) = true
    castlingAvailable(e8g8) = true
    castlingAvailable(e8c8) = true
    enPassantTargetSquare(white) = none
    enPassantTargetSquare(black) = none
    materialBalance = 0
    activeColor = white
    halfmoveCount = 0
  else
    dim as string symbol = "kqrbnp PNBRQK", a(1 to 6)
    z = InStr(s, " ") : a(1) = Left(s, z - 1) : s = Right(s, Len(s) - z)
    z = InStr(s, " ") : a(2) = Left(s, z - 1) : s = Right(s, Len(s) - z)
    z = InStr(s, " ") : a(3) = Left(s, z - 1) : s = Right(s, Len(s) - z)
    z = InStr(s, " ") : a(4) = Left(s, z - 1) : s = Right(s, Len(s) - z)
    z = InStr(s, " ") : a(5) = Left(s, z - 1) : s = Right(s, Len(s) - z)
    a(6) = s
    
    x = 1
    y = 8
    for z = 1 to Len(a(1))
      if a(1)[z - 1] = Asc("/") then
        x = 1
        y -= 1
      end if
      if (a(1)[z - 1] >= Asc("1")) andalso (a(1)[z - 1] <= Asc("8")) then
        do while a(1)[z - 1] > Asc("0")
          board(10 * x + y) = cNothingValue
          a(1)[z - 1] -= 1
          x += 1
        loop
      end if
      if InStr(symbol, chr(a(1)[z - 1])) > 0 then
        select case InStr(symbol, chr(a(1)[z - 1])) - 7
          case -6 : board(10 * x + y) = black * cKingValue
          case -5 : board(10 * x + y) = black * cQueenValue
          case -4 : board(10 * x + y) = black * cRookValue
          case -3 : board(10 * x + y) = black * cBishopValue
          case -2 : board(10 * x + y) = black * cKnightValue
          case -1 : board(10 * x + y) = black * cPawnValue
          case +1 : board(10 * x + y) = white * cPawnValue
          case +2 : board(10 * x + y) = white * cKnightValue
          case +3 : board(10 * x + y) = white * cBishopValue
          case +4 : board(10 * x + y) = white * cRookValue
          case +5 : board(10 * x + y) = white * cQueenValue
          case +6 : board(10 * x + y) = white * cKingValue
        end select
        x += 1
      end if
    next z
    for y = 8 to 1 step -1
      for x = 1 to 8
        materialBalance += board(10 * x + y)
      next x
    next y
    activeColor = Iif(a(2) = "w", white, black)
    castlingAvailable(e1g1) = (InStr(a(3), "K") > 0)
    castlingAvailable(e1c1) = (InStr(a(3), "Q") > 0)
    castlingAvailable(e8g8) = (InStr(a(3), "k") > 0)
    castlingAvailable(e8c8) = (InStr(a(3), "q") > 0)
    if Len(a(4)) = 2 then
      enPassantTargetSquare(-1 * activeColor) = 10 * (a(4)[0] - 96) + (a(4)[1] - 48)
    else
      enPassantTargetSquare(-1 * activeColor) = none
    end if
    enPassantTargetSquare(activeColor) = none
    halfmoveCount = 2 * (ValInt(a(6)) - 1) + Iif(activeColor = black, 1, 0)
  end if
  
  oneKingCaptured = false
end sub ' Initialize

sub TBoard.MovePiece(_
  byval aFromSquare as byte, _
  byval aToSquare as byte, _
  byval aPromotion as byte = cQueenValue _
)
  if Abs(board(aFromSquare)) = cKingValue then
    if Sgn(board(aFromSquare)) = white then
      castlingAvailable(e1g1) = false
      castlingAvailable(e1c1) = false
    else
      castlingAvailable(e8g8) = false
      castlingAvailable(e8c8) = false
    end if
  end if
  
  if Abs(board(aFromSquare)) = cRookValue then
    select case aFromSquare
      case 81 : castlingAvailable(e1g1) = false
      case 11 : castlingAvailable(e1c1) = false
      case 88 : castlingAvailable(e8g8) = false
      case 18 : castlingAvailable(e8c8) = false
    end select
  end if
  
  if Abs(board(aToSquare)) = cRookValue then
    select case aToSquare
      case 81 : castlingAvailable(e1g1) = false
      case 11 : castlingAvailable(e1c1) = false
      case 88 : castlingAvailable(e8g8) = false
      case 18 : castlingAvailable(e8c8) = false
    end select
  end if
  
  if Abs(board(aToSquare)) = cKingValue then oneKingCaptured = true
  
  if (Abs(aToSquare - aFromSquare) = 2) and (Abs(board(aFromSquare)) = cPawnValue) then
    enPassantTargetSquare(activeColor) = aFromSquare + cPawnMove(1) * activeColor
  else
    enPassantTargetSquare(activeColor) = none
  end if
  
  if (aToSquare = enPassantTargetSquare(- 1 * activeColor)) and (board(aFromSquare) = cPawnValue * activeColor) then
     enPassantTargetSquare(activeColor) = none
     MovePiece(aFromSquare, (aToSquare \ 10) * 10 + aFromSquare mod 10)
     activeColor = -1 * activeColor
     halfmoveCount -= 1
     aFromSquare = (aToSquare \ 10) * 10 + aFromSquare mod 10
  end if
  
  if ((aFromSquare >= 51) and (aFromSquare <= 58)) _
  andalso (_
    (aToSquare = 71) or _
    (aToSquare = 31) or _
    (aToSquare = 78) or _
    (aToSquare = 38) _
 ) _
  andalso (Abs(board(aFromSquare)) = cKingValue) then
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
    
    if aToSquare \ 10 = 7 then
      aFromSquare = (aFromSquare mod 10) + 80
      aToSquare = aFromSquare - 20
    else
      aFromSquare = (aFromSquare mod 10) + 10
      aToSquare = aFromSquare + 30
    end if
    
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
  else
    materialBalance = materialBalance - board(aToSquare)
    board(aToSquare) = board(aFromSquare)
    board(aFromSquare) = cNothingValue
    if (_
      (aToSquare mod 10 = 1) or _
      (aToSquare mod 10 = 8) _
   ) and (Abs(board(aToSquare)) = cPawnValue) then
      board(aToSquare) = aPromotion * activeColor
      materialBalance = materialBalance + (aPromotion - cPawnValue) * activeColor
    end if
  end if
  
  activeColor = -1 * activeColor
  halfmoveCount += 1
end sub ' MovePiece

sub TBoard.AddMoveToList(byval aFromSquare as byte, byval aToSquare as byte)
  if (oneKingCaptured andalso Abs(board(aToSquare)) = cKingValue) _
  or (list.count = moveListSize) then exit sub
  
  list.count += 1
  list.array(list.count).fromSquare = aFromSquare
  list.array(list.count).toSquare = aToSquare
end sub ' AddMoveToList

sub TBoard.GenerateMoves()
  dim as integer aFromSquare, aToSquare
  dim as integer i, j, k, l
  
  list.count = 0
  
  for j = 1 to 8
    for k = 1 to 8
      aFromSquare = j * 10 + k
      if board(aFromSquare) <> cNothingValue then
        select case board(aFromSquare) * activeColor
        
        case cPawnValue
          aToSquare = aFromSquare + cPawnMove(1) * activeColor
          
          if board(aToSquare) = cNothingValue then
            AddMoveToList(aFromSquare, aToSquare)
            if (activeColor = white) and (k = 2) _
            or (activeColor = black) and (k = 7) then
              aToSquare = aFromSquare + cPawnMove(2) * activeColor
              if (board(aToSquare) = cNothingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            end if
          end if
          
          for i = 3 to 4
            aToSquare = aFromSquare + cPawnMove(i) * activeColor
            if ((-1 * board(aToSquare) * activeColor >= cPawnValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue)) _
            or (aToSquare = enPassantTargetSquare(-1 * activeColor)) then
              AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cKnightValue
          for i = 1 to 8
            aToSquare = aFromSquare + cKnightMove(i)
            if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue) then
              AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cBishopValue
          for i = 1 to 4
            aToSquare = aFromSquare
            do
              aToSquare += cBishopMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
          
        case cRookValue
          for i = 1 to 4
            aToSquare = aFromSquare
            do
              aToSquare += cRookMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
          
        case cKingValue
          for i = 1 to 8
            aToSquare = aFromSquare + cKingMove(i)
            if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
            and (-1 * board(aToSquare) * activeColor <= cKingValue) then
              l = 0
              do
                l += 1
              loop until (-1 * board(aToSquare + cKingMove(l)) * activeColor = cKingValue) or (l = 8)
              if l = 8 then AddMoveToList(aFromSquare, aToSquare)
            end if
          next i
          
        case cQueenValue
          for i = 1 to 8
            aToSquare = aFromSquare
            do
              aToSquare += cKingMove(i)
              if (-1 * board(aToSquare) * activeColor >= cNothingValue) _
              and (-1 * board(aToSquare) * activeColor <= cKingValue) then
                AddMoveToList(aFromSquare, aToSquare)
              end if
            loop until board(aToSquare) <> cNothingValue
          next i
        end select
      end if
    next k
  next j
end sub ' GenerateMoves

sub TBoard.GenerateCastling(byval aCastling as tCastling)
  dim as byte kingFrom, kingStep, rookFrom, i
  
  select case as const aCastling
    case e1g1
      kingFrom = 51
      kingStep = +10
      rookFrom = 81
    case e1c1
      kingFrom = 51
      kingStep = -10
      rookFrom = 11
    case e8g8
      kingFrom = 58
      kingStep = +10
      rookFrom = 88
    case e8c8
      kingFrom = 58
      kingStep = -10
      rookFrom = 18
  end select
  
  i = kingFrom + kingStep
  do
    if board(i) <> cNothingValue then exit sub
    i += kingStep
  loop until i = rookFrom
  AddMoveToList(kingFrom, kingFrom + 2 * kingStep)
end sub

#macro CastlingProcedure(m)
if castlingAvailable((m)) andalso castlingLicence((m)) then
  GenerateCastling((m))
end if
#endmacro

sub TBoard.GenerateAllMoves()
  CheckCastlingLicence(activeColor)
  GenerateMoves()
  if activeColor = white then
    CastlingProcedure(e1g1)
    CastlingProcedure(e1c1)
  else
    CastlingProcedure(e8g8)
    CastlingProcedure(e8c8)
  end if
end sub ' GenerateAllMoves

sub TBoard.CheckCastlingLicence(byval aColor as byte)
  dim c as tCastling
  dim y as byte = Iif(aColor = white, 1, 8)
  for c = e1g1 to e8c8
    castlingLicence(c) = true
  next c
  
  activeColor = -1 * activeColor
  GenerateMoves()
  
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if list.array(i).toSquare mod 10 = y then
      select case list.array(i).toSquare \ 10
        case is = 5
          castlingLicence(e1g1) = false
          castlingLicence(e1c1) = false
          castlingLicence(e8g8) = false
          castlingLicence(e8c8) = false
        case is > 5
          castlingLicence(e1g1) = false
          castlingLicence(e8g8) = false
        case is < 5
          castlingLicence(e1c1) = false
          castlingLicence(e8c8) = false
      end select
    end if
  next i
end sub ' CheckCastlingLicence

function TBoard.PositionValue(byval aColor as byte) as short
  return materialBalance * aColor
end function ' PositionValue

function TBoard.Text() as string
  dim as string s = _
  "+ A B C D E F G H " & EOL & _
  "8                 " & EOL & _
  "7                 " & EOL & _
  "6                 " & EOL & _
  "5                 " & EOL & _
  "4                 " & EOL & _
  "3                 " & EOL & _
  "2                 " & EOL & _
  "1                 " & EOL & _
  "+ A B C D E F G H "
  for y as integer = 8 to 1 step -1
    for x as integer = 1 to 8
      select case Abs(board(10 * x + y))
        case cNothingValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("-")
        case cPawnValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("P")
        case cKnightValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("N")
        case cBishopValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("B")
        case cRookValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("R")
        case cQueenValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("Q")
        case cKingValue
          s[20 * (8 - y) + 2 * x + 20] = Asc("K")
      end select
      if board(10 * x + y) < cNothingValue then
        s[20 * (8 - y) + 2 * x + 20] += 32
      end if
    next x
  next y
  return s
end function ' Text

function TBoard.Check() as byte
  activeColor = -1 * activeColor
  GenerateMoves()
  
  activeColor = -1 * activeColor
  for i as integer = 1 to list.count
    if Abs(board(list.array(i).toSquare)) = cKingValue then
      return true
    end if
  next i
  
  return false
end function ' Check

DefaultPlacement:
data -010, -007, -006, -019, -126, -006, -007, -010
data -002, -002, -002, -002, -002, -002, -002, -002
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data -000, -000, -000, -000, -000, -000, -000, -000
data +002, +002, +002, +002, +002, +002, +002, +002
data +010, +007, +006, +019, +126, +006, +007, +010
