
#include once "pcre.bi"

namespace uci

declare function IsCommandQuit(cmd as const string) as boolean
declare function IsCommandUCI(cmd as const string) as boolean
declare function IsCommandReady(cmd as const string) as boolean
declare function IsCommandNewGame(cmd as const string) as boolean
declare function IsCommandPosition(cmd as const string) as boolean
declare function IsCommandStartPos(cmd as const string) as boolean
declare function IsCommandFEN(cmd as const string) as boolean
declare function GetFEN(cmd as const string, byref aFEN as string) as boolean
declare function IsCommandMoves(cmd as const string) as boolean
declare function GetMoves(cmd as const string, aMoves() as string) as boolean
declare function IsCommandGo(cmd as const string) as boolean

const MOVE_PATTERN = _
  "[a-h][1-8][a-h][1-8][nbrq]*"
const FEN_RECORD_PATTERN = _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & "/" & _
  "[1-8BKNPQRbknpqr]+" & _
  "\s" & "[wb]" & _
  "\s" & "([KQkq]+|\-)" & _
  "\s" & "([a-h][36]|\-)" & _
  "\s" & "\d+" & _
  "\s" & "[1-9]\d*"

function ExtractStrings(byval aPattern as string, byval aBuffer as string, aArray() as string, aPreserve as boolean = false) as integer
  const OVECCOUNT = 300
  dim as zstring ptr error_
  dim as integer error_offset
  dim as integer rc, i
  dim as integer ovector(OVECCOUNT - 1)
  dim as integer result
  dim as pcre ptr re
  
  if not aPreserve then
    erase aArray
  end if
  
  result = 0
  
  re = pcre_compile(aPattern, 0, @error_, @error_offset, NULL)
  if re = NULL then
    return result
  end if
  
  i = 0
  do
    rc = pcre_exec(re, NULL, strptr(aBuffer), len(aBuffer), i, 0, @ovector(0), OVECCOUNT)
    if rc > 0 then
      redim preserve aArray(lbound(aArray) to ubound(aArray) + 1)
      aArray(ubound(aArray)) = mid(aBuffer, ovector(0) + 1, ovector(1) - ovector(0))
      result += 1
      i = ovector(1)
    end if
  loop while rc >= 0

  return result
end function

function IsCommandQuit(cmd as const string) as boolean
  return cbool(cmd = "quit")
end function

function IsCommandUCI(cmd as const string) as boolean
  return cbool(cmd = "uci")
end function

function IsCommandReady(cmd as const string) as boolean
  return cbool(cmd = "isready")
end function

function IsCommandNewGame(cmd as const string) as boolean
  return cbool(cmd = "ucinewgame")
end function

function IsCommandPosition(cmd as const string) as boolean
  return cbool(InStr(cmd, "position") = 1)
end function

function IsCommandStartPos(cmd as const string) as boolean
  return cbool(InStr(cmd, "startpos"))
end function

function IsCommandFEN(cmd as const string) as boolean
  return cbool(InStr(cmd, "fen"))
end function

function GetFEN(cmd as const string, byref aFEN as string) as boolean
  dim ss() as string
  dim result as integer = ExtractStrings(FEN_RECORD_PATTERN, cmd, ss())
  if result = 1 then
    aFEN = ss(0)
  else
    aFEN = ""
  end if
  return cbool(result = 1)
end function

function IsCommandMoves(cmd as const string) as boolean
  return cbool(InStr(cmd, "moves"))
end function

function GetMoves(cmd as const string, aMoves() as string) as boolean
  dim result as integer = ExtractStrings(MOVE_PATTERN, cmd, aMoves())
  return cbool(result)
end function

function IsCommandGo(cmd as const string) as boolean
  return cbool(InStr(cmd, "go") = 1)
end function

end namespace
